function hOut = patchplot(x, y, scores, varargin)


% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

if nargin < 3, scores = []; end
if nargin < 2, y = []; end
if isstr(y), varargin = [{y scores} varargin]; y = []; scores = []; end
if isstr(scores), varargin = [{scores} varargin]; scores = []; end

if size(x, 2) == 2 & (isempty(y) | isempty(scores))
	if isempty(scores), scores = y; end
	y = x(:, 2);
	x = x(:, 1);
end
if isempty(y), [x y] = deal(y, x); end
if isempty(x), x = 1:length(y); end
x = x(:);
y = y(:);
scores = scores(:);

if length(x) ~= length(y), error('mismatched argument lengths'), end
if ~isempty(scores) & length(scores) ~= length(y), error('mismatched argument lengths'), end

if ~ishold, cla, end

[keys vals err] = keysvals(varargin); error(err)

ind = find(strcmp(lower(keys), 'vertices'));
if isempty(ind), vertices = 4; else vertices = vals{max(ind)}; end
vals(ind) = []; keys(ind) = [];

ind = find(strcmp(lower(keys), 'size'));
if isempty(ind), siz = [1 1.618]; else, siz = vals{max(ind)}; end
vals(ind) = []; keys(ind) = [];
siz = siz * 0.01;
if prod(size(siz)) == 2, siz = repmat(siz(:)', length(x), 1);
elseif prod(size(siz)) == 1, siz = repmat([siz siz], length(x), 1);
elseif prod(size(siz)) == length(siz)
	siz = [siz(:) siz(:)*1.618];
end
if ~isequal(size(siz), [length(x) 2]), error('''Size'' value should have one or two columns, and either one row or one row per point'), end

ec = 0;
ind = find(strcmp(lower(keys), 'edgecolor'));
if ~isempty(ind) & iscellstr(vals(max(ind)))
	if strcmp(lower(vals{max(ind)}), 'contrast'), ec = 1; vals(ind) = []; keys(ind) = []; end
end

bad = isinf(x) | isinf(y) | isnan(x) | isnan(y);
x(bad) = []; y(bad) = []; siz(bad, :) = [];
if ~isempty(scores), scores(bad) = []; end

ax = gca;

xr = [min(x) max(x)];
if strcmp(lower(get(ax, 'xlimmode')), 'manual'), xr = get(ax, 'xlim'); end
yr = [min(y) max(y)];
if strcmp(lower(get(ax, 'ylimmode')), 'manual'), yr = get(ax, 'ylim'); end
cr = get(ax, 'clim');
if strcmp(lower(get(ax, 'climmode')), 'auto') & ~isempty(scores)
	cr = scores(~isinf(scores)); cr = [min(cr) max(cr)];
end
if isempty(scores), scores = zeros(size(x)); end

if any(strcmp('manual', get(ax, {'plotboxaspectratiomode' 'dataaspectratiomode'})))
	ar = get(ax, 'plotboxaspectratio');
	ar = ar(1:2);
else
	oldu = get(ax, 'units');
	set(ax, 'units', 'pixels')
	ar = get(ax, 'position');
	set(ax, 'units', oldu)
	ar = ar(3:4);
end

xrad = siz(:, 1)' * diff(xr)/2;
yrad = siz(:, 2)' * diff(yr)/2;
yrad = yrad * ar(1) / ar(2);

th = -pi/2 + linspace(0, 2 * pi, vertices + 1)';
xd = cos(th(1:end-1)) * xrad;
yd = sin(th(1:end-1)) * yrad;

x = repmat(x(:)', vertices, 1) + xd;
y = repmat(y(:)', vertices, 1) + yd;
scores = repmat(scores(:)', size(x, 1), 1);

opts = [keys;vals];
h = patch(x, y, scores, 'parent', ax, 'edgecolor', 'none', 'facecolor', 'flat', opts{:});
set(ax, 'clim', cr)
if ec
	fig = get(ax, 'parent');
	c = mean(scores(:));
	c = min(1, max(0, (c - cr(1)) / diff(cr)));
	cmap = get(fig, 'colormap');
	c = cmap(round(1 + c * (size(cmap, 1) - 1)), :);
	set(h, 'edgecolor', ContrastingColour(c))
end
if nargout, hOut = h; else figure(gcf), drawnow, end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cOut = ContrastingColour(cIn)

cOut = 1 - cIn;
cIn = rgb2hsv(cIn);
cOut = rgb2hsv(cOut);
cOut(:, 2) = 1;
cOut = hsv2rgb(cOut);
