function s = texprintf(fmtstr, varargin)
% s = texprintf(fmt, ...)
% 
% Exactly like sprintf, except that the \ character
% is no longer an escape character (therefore, this
% is very useful for compiling TeX strings, because
% all the hundreds of \ do not need to be doubled).
%
% To use a conventional sprintf escape character,
% prefix the normal code with a % - for example:
% 
% 	s = texprintf('\begin{itemize}%\n');
% 	for i = 1:3
% 		s = [s texprintf('%\t\item \bm{\hat{\theta}}_{%d}%\n', i)];
% 	end
% 	s = [s texprintf('\end{itemize}%\n')];
% 	disp(s)

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

dummySequence = char([1 2]);
fmtstr = strrep(fmtstr, dummySequence, '');
fmtstr = strrep(fmtstr, '%\', dummySequence);
fmtstr = strrep(fmtstr, '\', '\\');
fmtstr = strrep(fmtstr, dummySequence, '\');
s = sprintf(fmtstr, varargin{:});
