function coords = translateaxescoords(ax, coords, fromUnits, toUnits)
% COORDS = TRANSLATEAXESCOORDS(AX, COORDS, FROM_UNITS, TO_UNITS)
% 
% Translates the first two columns of COORDS, which are x and y
% coordinates on the axes AX, between any of the following units:
% 
%     'inches', 'centimeters', 'normalized',
%     'points', 'pixels', 'characters', 'data'
% 
% See also GETTRUEAXESPOSITION

% Part of PFCMP version 2.5.41, an extension to the PSIGNIFIT toolbox for MATLAB versions 5 and up.
% The license and NO WARRANTY statement for PSIGNIFIT also applies to this release - see psych_legal.m
% Copyright (c) J.Hill 2002.
% mailto:psignifit@bootstrap-software.org
% http://www.bootstrap-software.org/psignifit/

if nargin < 1, ax = gca; end
if nargin < 2, coords = [0.5 0.5]; end
if nargin < 3, fromUnits = 'data'; end
if nargin < 4
	if strcmp(lower(fromUnits), 'pixels')
		toUnits = 'data';
	else
		toUnits = 'pixels';
	end
end

units = lower({fromUnits, toUnits}); rect = {};

ischaracters = strcmp(units, 'characters');
units(ischaracters) = {'pixels'};
if any(ischaracters)
	oldUnits = get(ax, 'units');
	set(ax, 'units', 'characters')
	characterDims = get(ax, 'position');
	set(ax, 'units', oldUnits)
	characterDims = characterDims(3:4);
end

isdata = strcmp(units, 'data');
if any(isdata)
	islog = strcmp(get(ax, {'xscale', 'yscale'}), 'log');
	isreverse = strcmp(get(ax, {'xdir', 'ydir'}), 'reverse');
	xl = get(ax, 'xlim');
	if isreverse(1), xl = fliplr(xl); end
	if islog(1), xl = log(xl); end
	yl = get(ax, 'ylim');
	if isreverse(2), yl = fliplr(yl); end
	if islog(2), yl = log(yl); end
	dataRect = [xl(1) yl(1) diff(xl) diff(yl)];
end
if isdata(1)
	if islog(1), coords(:, 1) = log(coords(:, 1)); end
	if islog(2), coords(:, 2) = log(coords(:, 2)); end
end

pbar = get(ax, 'plotboxaspectratio');
pbar = pbar([1 2]) ./ pbar([2 1]);
constrained = any(strcmp(get(ax, {'plotboxaspectratiomode', 'dataaspectratiomode'}), 'manual'));

for i = 1:2
	u = units{i};
	switch units{i}
	case 'data'
		rect{i} = dataRect;
	case 'normalized'
		rect{i} = [0 0 1 1];
	otherwise
		oldUnits = get(ax, 'units');
		if ~strcmp(u, oldUnits), set(ax, 'units', u), end
		dims = get(ax, 'position'); dims = dims(3:4);
		if ~strcmp(u, oldUnits), set(ax, 'units', oldUnits), end
		if ischaracters(i), charPerPixel = characterDims ./ dims; end
		if constrained, dims = min(dims, dims([2 1]) .* pbar); end
		if ischaracters(i), dims = dims .* charPerPixel; end		
		rect{i} = [0 0 dims];
	end
end

mult = rect{2}(3:4) ./ rect{1}(3:4);
add = rect{2}(1:2) - rect{1}(1:2) .* mult;
coords(:, 1) = coords(:, 1) * mult(1) + add(1);
coords(:, 2) = coords(:, 2) * mult(2) + add(2);
if isdata(2)
	if islog(1), coords(:, 1) = exp(coords(:, 1)); end
	if islog(2), coords(:, 2) = exp(coords(:, 2)); end
end
